import pytest
import random
import datetime
from model_bakery import baker
from django.utils import timezone as tz
from datetime import timedelta
import pytz
from aircox.models import Diffusion


class TestDiffusionQuerySet:
    @pytest.mark.django_db
    def test_episode_by_obj(self, diff_episode_var):
        key_episode = diff_episode_var[0]
        diff_with_key_episode = diff_episode_var[2]
        diff_without_key_episode = diff_episode_var[3]
        result = Diffusion.objects.episode(episode=key_episode)

        assert all(diff in result for diff in diff_with_key_episode)
        assert all(diff not in result for diff in diff_without_key_episode)

    @pytest.mark.django_db
    def test_episode_by_id(self, diff_episode_var):
        key_episode = diff_episode_var[0]
        diff_with_key_episode = diff_episode_var[2]
        diff_without_key_episode = diff_episode_var[3]
        result = Diffusion.objects.episode(id=key_episode.id)

        assert all(diff in result for diff in diff_with_key_episode)
        assert all(diff not in result for diff in diff_without_key_episode)

    @pytest.mark.django_db
    def test_on_air(self, episodes):
        for episode in episodes:
            baker.make(
                Diffusion,
                episode=episode,
                type=random.choice(
                    [Diffusion.TYPE_ON_AIR, Diffusion.TYPE_UNCONFIRMED]
                ),
            )
        assert all(
            diff.type == Diffusion.TYPE_ON_AIR
            for diff in Diffusion.objects.on_air()
        )

    # TODO test si il y a doublon qu'un seul objet soit récupéré.
    @pytest.mark.django_db
    def test_now_test(self, diff_time_var):
        now_diff, result = diff_time_var[4], Diffusion.objects.now(now=tz.now(), order=True) 
        assert len(result) == 1 and result[0] == now_diff and now_diff.start <= tz.now() <= now_diff.end

    @pytest.mark.django_db
    def test_date_without_date_arg(self, diff_time_var):
        before_diff, now_diff, after_diff = diff_time_var[3:6]
        result = Diffusion.objects.date()
        assert len(result) == 3
        assert before_diff, now_diff and after_diff in result

    @pytest.mark.django_db
    def test_date_with_date_arg(self, diff_time_var):
        tomorrow_date = datetime.date.today() + timedelta(days=1)
        tomorrow_diff = diff_time_var[6]
        assert Diffusion.objects.date(date=tomorrow_date).count() == 1
        assert Diffusion.objects.date(date=tomorrow_date)[0] == tomorrow_diff

    @pytest.mark.django_db
    def test_at_datetimeobj(self, diff_time_var):
        now_datetime, now_diff = diff_time_var[1], diff_time_var[4]
        assert Diffusion.objects.at(date=now_datetime).count() == 1
        assert Diffusion.objects.at(date=now_datetime)[0] == now_diff

    @pytest.mark.django_db
    def test_at_dateobj(self, diff_time_var):
        now_date=diff_time_var[0]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        result = Diffusion.objects.at(date=now_date)
        assert len(result) == 3
        assert before_diff, now_diff and after_diff in result

    @pytest.mark.django_db
    def test_after_without_arg(self, diff_time_var):
        yesterday_diff=diff_time_var[2]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        tomorrow_diff=diff_time_var[6]
        from_now_diff = Diffusion.objects.after()
        assert set([tomorrow_diff, after_diff, now_diff]).issubset(set(from_now_diff))
        assert before_diff not in from_now_diff and yesterday_diff not in from_now_diff

    @pytest.mark.django_db
    def test_after_with_datetime_arg(self, diff_time_var):
        now_datetime=diff_time_var[1]
        yesterday_diff=diff_time_var[2]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        tomorrow_diff=diff_time_var[6]
        from_now_diff = Diffusion.objects.after(date=now_datetime)
        assert set([tomorrow_diff, after_diff, now_diff]).issubset(set(from_now_diff))
        assert before_diff not in from_now_diff and yesterday_diff not in from_now_diff

    @pytest.mark.django_db
    def test_after_with_date_arg(self, diff_time_var):
        now_date=diff_time_var[0]
        yesterday_diff=diff_time_var[2]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        tomorrow_diff=diff_time_var[6]
        from_today_diff = Diffusion.objects.after(date=now_date)
        assert set([tomorrow_diff, after_diff, now_diff, before_diff]).issubset(set(from_today_diff))
        assert yesterday_diff not in from_today_diff

    @pytest.mark.django_db
    def test_before_without_arg(self, diff_time_var):
        yesterday_diff=diff_time_var[2]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        tomorrow_diff=diff_time_var[6]
        before_now_diff = Diffusion.objects.before()
        assert tomorrow_diff not in before_now_diff and after_diff not in before_now_diff
        assert set([now_diff, before_diff, yesterday_diff]).issubset(set(before_now_diff))
 

    @pytest.mark.django_db
    def test_before_with_datetime_arg(self, diff_time_var):
        now_datetime=diff_time_var[1]
        yesterday_diff=diff_time_var[2]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        tomorrow_diff=diff_time_var[6]
        before_now_diff = Diffusion.objects.before(date=now_datetime)
        assert tomorrow_diff not in before_now_diff and after_diff not in before_now_diff
        assert set([now_diff, before_diff, yesterday_diff]).issubset(set(before_now_diff))

    @pytest.mark.django_db
    def test_before_with_date_arg(self, diff_time_var):
        now_date=diff_time_var[0]
        yesterday_diff=diff_time_var[2]
        before_diff=diff_time_var[3]
        now_diff=diff_time_var[4]
        after_diff=diff_time_var[5]
        tomorrow_diff=diff_time_var[6]
        before_today_diff = Diffusion.objects.before(date=now_date)
        assert all(diff not in before_today_diff for diff in [tomorrow_diff, after_diff, now_diff])
        assert yesterday_diff in before_today_diff

    @pytest.mark.django_db
    def test_range(self, episodes, diff_time_var):
        start, end = diff_time_var[3].start, diff_time_var[5].end
        yesterday_diff, before_diff, now_diff, after_diff, tomorrow_diff = diff_time_var[2:7]
        overlaping_start_diff = baker.make(Diffusion, episode=episodes[0], start=start - timedelta(minutes=30), end=end)
        overlaping_end_diff = baker.make(Diffusion, episode=episodes[0], start=start, end=end + timedelta(minutes=30))
        range_diff = Diffusion.objects.range(start=start, end=end)
        assert set([before_diff, after_diff, now_diff]).issubset(range_diff)
        assert not set([overlaping_start_diff, overlaping_end_diff, tomorrow_diff, yesterday_diff]).intersection(range_diff)

#TODO CLEAN ce qu'il y a en dessous et terminer les tests de diffusion.py

class TestDiffusion:
    @pytest.mark.django_db
    def test__str__initial_diff(self, diff_with_rerun):
        (
            initial_diff,
            _,
        ) = diff_with_rerun
        initial_diff.episode.title = "Episode title"
        initial_diff.start = datetime.datetime(
            2023, 5, 12, 13, 0, tzinfo=pytz.UTC
        )
        assert (
            initial_diff.__str__() == "Episode title - 2023/05/12 13:00+0000"
        )

    @pytest.mark.django_db
    def test__str__rerun_diff(self, diff_with_rerun):
        (
            initial_diff,
            rerun_diff,
        ) = diff_with_rerun
        initial_diff.episode.title = "Episode title"
        rerun_diff.start = datetime.datetime(
            2023, 5, 12, 13, 0, tzinfo=pytz.UTC
        )
        assert (
            rerun_diff.__str__()
            == "Episode title - 2023/05/12 13:00+0000 (rerun)"
        )

    # dans la méthode self._initial[valeur] est un dictionnaire qui stocke les valeurs initiales des champs de l'objet au moment de sa création
    # save
    # si diff est initial et si son épisode a été modifié depuis la création initiale
    # la méthode save update l'épisode et le programme sur paramètre initial (rerun_set) de la rediff (qui représente l'objet diff initial).
    """ je ne comprends pas rerun_set.update, que la méthode l'update ou pas, on accède quand même à l'épisode mis à jour sur la diff initiale"""

    @pytest.mark.django_db
    def test_save_rerun_diff(self, episodes, diff_with_rerun):
        (
            initial_diff,
            rerun_diff,
        ) = diff_with_rerun
        initial_diff.episode = episodes[1]
        initial_diff.program = episodes[1].program
        initial_diff.save()
        assert rerun_diff.initial.episode == episodes[1]
        assert rerun_diff.initial.program == episodes[1].program

    @pytest.mark.django_db
    def test_save_rerun(self, diff_with_rerun):
        (
            initial_diff,
            rerun_diff,
        ) = diff_with_rerun
        rerun_diff.save_rerun()
        assert rerun_diff.episode == initial_diff.episode

    @pytest.mark.django_db
    def test_save_initial(self, diff_with_rerun):
        (
            initial_diff,
            rerun_diff,
        ) = diff_with_rerun
        initial_diff.save_initial()
        assert initial_diff.program == initial_diff.episode.program

    @pytest.mark.django_db
    def test_duration(self, diff):
        assert diff.duration == datetime.timedelta(seconds=1800)

    @pytest.mark.django_db
    def test_date(self, diff):
        assert diff.date == datetime.date(2023, 5, 12)

    @pytest.mark.django_db
    def test_local_start(self, diff):
        assert diff.local_start == datetime.datetime(
            2023, 5, 12, 13, 00, tzinfo=pytz.UTC
        )

    @pytest.mark.django_db
    def test_local_end(self, diff):
        assert diff.local_end == datetime.datetime(
            2023, 5, 12, 13, 30, tzinfo=pytz.UTC
        )

    @pytest.mark.django_db
    def test_is_now(self, diff):
        diff.type = Diffusion.TYPE_ON_AIR
        diff.start = tz.now()
        diff.end = tz.now() + timedelta(minutes=30)
        assert diff.is_now == True

    @pytest.mark.django_db
    def test_is_live(self, diff):
        diff.type = Diffusion.TYPE_ON_AIR
        diff.episode.sound_set == None
        assert diff.is_live == True

    # on ne peut pas changer l'heure de diff ?
    # souci avec save
    # un autre souci ailleurs : est overlaping devrait être testé partout aussi ?
